//Configuration Sketch - Version 2.00 - www.cheap-thermocam.com

//Copyright 2013 by Max Ritter 
//Concerning any questions, contact mail@maxritter.net

//This sketch writes the required EEPROM settings for the Cheap-Thermocam into the MLX90614
//Transmit this to the arduino, connect the MLX90614 and open the Serial window under Tools -> Serial Monitor
//If you see the success prompt at the end, plug off the arduino and transmit the main sketch onto it
//You can also use the attached TemperatureRead Sketch to test the functionality of your flashed sensor

//You need to put the included I2CMaster library into your Arduino/library folder
#include <i2cmaster.h>
#include <EEPROM.h>

int dev =  0x00;
int version = 0;

void setup()
{
//Make sure you set the Baudrate inside the Serial Window to 9600
Serial.begin(9600);
Serial.println("This program will change the EEPROM settings of your");
Serial.println("MLX90614 BCI or DCI sensor to work best with the Cheap-Thermocam.");
Serial.println("");
Serial.println("PLEASE NOTE DOWN CAREFULLY THE FOLLOWING VALUES !");
Serial.println("They are required to restore if anything goes wrong");
Serial.println("");

//Initialising the I2C Bus and activating the internal pullup resistors
i2c_init();
PORTC = (1 << PORTC4) | (1 << PORTC5); 

//Reading the previous values
Serial.print("Filter settings: ");
Serial.println(I2CRead(0x25));

Serial.print("Maximum temp: ");
Serial.println(I2CRead(0x20));

Serial.print("Minimum temp: ");
Serial.println(I2CRead(0x21));

Serial.println("");
Serial.println("SEND 'b' TO FLASH THE BCI VERSION");
Serial.println("OR 'd' TO FLASH THE DCI VERSION !");
Serial.println("");
//Wait for keyboard input
while (Serial.available() == 0) {
}  
//Selecting the right version
char inChar = Serial.read();
if(inChar == 'b'){
  Serial.println("-> The BCI version will now be flashed..");
  version = 1;
}

else if(inChar == 'd'){
  Serial.println("-> The DCI version will now be flashed..");
  Serial.println("");
  version = 2;
}
//Doing the required steps
EraseSettings();
WriteNewSettings();
CheckSettings();
SetArduinoEEPROM();
}


void loop()
{
}

void EraseSettings(){
  //WRITE TO EEPROM, FIRST: ERASE OLD STUFF
  Serial.println("*1: Erasing old EEPROM settings");
  Serial.println("");

  Serial.println("Erasing filter settings..");
  I2CWrite(0x25, 0x00, 0x00, 0x83);

  Serial.println("Erasing maximum temperature setting..");
  I2CWrite(0x20, 0x00, 0x00, 0x43);

  Serial.println("Erasing minimum temperature setting..");
  I2CWrite(0x21, 0x00, 0x00, 0x28);
}

void WriteNewSettings(){
  //WRITE TO EEPROM, THE NEW STUFF!
  Serial.println("");
  Serial.println("*2: Write new settings to EEPROM");
  Serial.println("");

  //Filter Settings = IIR 100, FIR, 100 -> 0.06s settling time & 100% Spike limit
  Serial.println("Writing new filter settings..");
  //BCI Filter settings, have a GAIN factor of 12.5
  if(version == 1){
    I2CWrite(0x25, 0x74, 0x9C, 0xA8);
  }
  //DCI Filter settings, have a GAIN factor of 100
  else if(version == 2){
    I2CWrite(0x25, 0x74, 0xB4, 0x70);
  }

  Serial.println("Writing new maximum temperature setting..");
  I2CWrite(0x20, 0x23, 0xFF, 0x21);

  Serial.println("Writing new minimum temperature setting..");
  I2CWrite(0x21, 0x5B, 0x4F, 0x59);
}

void CheckSettings(){
  boolean check = true;
  //CHECKING IF EVERYTHING IS OK
  Serial.println("");
  Serial.println("*3: Validating the new settings");
  Serial.println("");

  Serial.println("Checking filter settings..");
  //BCI version
  if(version == 1){
    if(I2CRead(0x25) != 40052) check = false;
  }
  //DCI version
  else if(version == 2){
    if(I2CRead(0x25) != 46196) check = false;
  }
  

  Serial.println("Checking maximum temperature setting..");
  if(I2CRead(0x20) != 65315) check = false;

  Serial.println("Checking minimum temperature setting..");
  if(I2CRead(0x21) != 20315) check = false;

  Serial.println("");
  
  //Success
  if(check == true){
    Serial.println("SUCCESS !");
    Serial.println("Please plug the arduino off from ANY POWER SOURCE now !");
  }
  
  //Anything went wrong, try again !
  else{
    Serial.println("ERROR ! Not all settings could be written !");  
    Serial.println("Re-run this sketch or check the connections !");  
  }
}

// Set the Arduino's EEPROM settings on register 0-3 to Zero in order
// to store the Servo position settings there later on
void SetArduinoEEPROM(){
  for(int i=0; i<=3;i++){
    EEPROM.write(i,0);
  }
}

// A method to write new values into a register
//For PEC Calculation have a look at : http://depa.usst.edu.cn/chenjq/www2/SDesign/JavaScript/CRCcalculation.htm
void I2CWrite(int adress, unsigned int LSB, unsigned int MSB, int PEC){
  i2c_start_wait(dev+I2C_WRITE);
  i2c_write(adress);
  i2c_write(LSB); 
  i2c_write(MSB);
  i2c_write(PEC);
  i2c_stop();  
  delay(100);
}

// A method to read values from a EEPROM adress
unsigned int I2CRead(int adress){
  i2c_start_wait(dev+I2C_WRITE);
  i2c_write(adress);
  i2c_rep_start(dev+I2C_READ);
  unsigned int LSB = i2c_readAck();
  unsigned int MSB = i2c_readAck();
  unsigned int pec = i2c_readNak();
  i2c_stop();
  unsigned int regValue = (((MSB) << 8) + LSB);
  return regValue;
  delay(100);
}

